use crate::hal::{deep_sleep, BackupHeader, BoardInteraction, Sensor};
use anyhow::{bail, Result};
use chrono::{DateTime, Utc};
use embedded_hal::digital::OutputPin;
use esp_idf_hal::gpio::{InputOutput, PinDriver};
use crate::config::{BoardHardware, PlantControllerConfig};
use crate::hal::battery::{BatteryInteraction, BatteryMonitor};
use crate::hal::esp::ESP;

pub struct Initial<'a> {
    pub(crate) general_fault: PinDriver<'a, esp_idf_hal::gpio::AnyIOPin, InputOutput>,
    pub(crate) esp: ESP<'a>,
    pub(crate) config: PlantControllerConfig,
}

impl BoardInteraction<'_> for Initial<'_> {
    fn get_esp<'a>(&mut self) -> &mut ESP<'a> {
        &mut self.esp
    }

    fn get_config(&mut self) -> &PlantControllerConfig {
        &self.config
    }

    fn get_battery_monitor(&mut self) -> Box<dyn BatteryInteraction> {
        let v = BatteryMonitor::Disabled {

        };
        Box::new(v) as Box<dyn BatteryInteraction>
    }

    fn set_charge_indicator(&mut self, charging: bool) -> Result<()> {
        bail!("Please configure board revision")
    }

    fn deep_sleep(&mut self, duration_in_ms: u64) -> ! {
        deep_sleep(duration_in_ms)
    }

    fn get_backup_info(&mut self) -> Result<BackupHeader> {
        bail!("Please configure board revision")
    }

    fn get_backup_config(&mut self) -> Result<Vec<u8>> {
        bail!("Please configure board revision")
    }

    fn backup_config(&mut self, bytes: &[u8]) -> Result<()> {
        bail!("Please configure board revision")
    }

    fn is_day(&self) -> bool {
        false
    }

    fn water_temperature_c(&mut self) -> Result<f32> {
        bail!("Please configure board revision")
    }

    fn tank_sensor_voltage(&mut self) -> Result<f32> {
        bail!("Please configure board revision")
    }
    fn light(&mut self, enable: bool) -> Result<()> {
        bail!("Please configure board revision")
    }

    fn pump(&mut self, plant: usize, enable: bool) -> Result<()> {
        bail!("Please configure board revision")
    }
    fn fault(&mut self, plant: usize, enable: bool) -> Result<()> {
        bail!("Please configure board revision")
    }
    fn measure_moisture_hz(&mut self, plant: usize, sensor: Sensor) -> Result<f32> {
        bail!("Please configure board revision")
    }

    fn general_fault(&mut self, enable: bool) {
        let _ = self.general_fault.set_state(enable.into());
    }

    fn factory_reset(&mut self) -> Result<()> {
        bail!("Please configure board revision")
    }

    fn get_rtc_time(&mut self) -> Result<DateTime<Utc>> {
        bail!("Please configure board revision")
    }

    fn set_rtc_time(&mut self, time: &DateTime<Utc>) -> Result<()> {
        bail!("Please configure board revision")
    }
    fn test_pump(&mut self, plant: usize) -> Result<()> {
        bail!("Please configure board revision")
    }

    fn test(&mut self) -> Result<()> {
        bail!("Please configure board revision")
    }
}